@@ -22,6 +22,7 @@
 #include "sbuild-chroot-config.h"
 #include "sbuild-chroot-directory.h"
 #include "sbuild-chroot-plain.h"
+#include "sbuild-chroot-custom.h"
 #include "sbuild-chroot-file.h"
 #ifdef SBUILD_FEATURE_BLOCKDEV
 #include "sbuild-chroot-block-device.h"
@@ -40,6 +41,7 @@
 #include "sbuild-chroot-facet-session.h"
 #include "sbuild-chroot-facet-session-clonable.h"
 #include "sbuild-chroot-facet-source.h"
+#include "sbuild-chroot-facet-userdata.h"
 #include "sbuild-fdstream.h"
 #include "sbuild-lock.h"
 
@@ -70,7 +72,7 @@ namespace
       emap(sbuild::chroot::CHROOT_CREATE,     N_("Chroot creation failed")),
       emap(sbuild::chroot::CHROOT_DEVICE,     N_("Device name not set")),
       // TRANSLATORS: %1% = chroot type name
-      emap(sbuild::chroot::CHROOT_TYPE,       N_("Unknown chroot type '%1%'")),
+      emap(sbuild::chroot::CHROOT_TYPE,       N_("Unknown chroot type ‘%1%’")),
       emap(sbuild::chroot::DEVICE_ABS,        N_("Device must have an absolute path")),
       emap(sbuild::chroot::DEVICE_LOCK,       N_("Failed to lock device")),
       emap(sbuild::chroot::DEVICE_NOTBLOCK,   N_("File is not a block device")),
@@ -86,6 +88,8 @@ namespace
       emap(sbuild::chroot::FILE_UNLOCK,       N_("Failed to discard file lock")),
       emap(sbuild::chroot::LOCATION_ABS,      N_("Location must have an absolute path")),
       emap(sbuild::chroot::NAME_INVALID,      N_("Invalid name")),
+      emap(sbuild::chroot::SCRIPT_CONFIG_CV,  N_("Could not set profile name from script configuration path ‘%1%’")),
+
       // TRANSLATORS: unlink refers to the C function which removes a file
       emap(sbuild::chroot::SESSION_UNLINK,    N_("Failed to unlink session file")),
       emap(sbuild::chroot::SESSION_WRITE,     N_("Failed to write session file")),
@@ -109,17 +113,22 @@ sbuild::chroot::chroot ():
   root_groups(),
   aliases(),
   preserve_environment(false),
+  default_shell(),
   environment_filter(SBUILD_DEFAULT_ENVIRONMENT_FILTER),
   mount_location(),
   original(true),
   run_setup_scripts(true),
-  script_config("default/config"),
+  script_config(),
+  profile("default"),
   command_prefix(),
   message_verbosity(VERBOSITY_NORMAL),
   facets()
 {
   add_facet(sbuild::chroot_facet_personality::create());
   add_facet(sbuild::chroot_facet_session_clonable::create());
+  add_facet(sbuild::chroot_facet_userdata::create());
+
+  set_profile(get_profile());
 }
 
 sbuild::chroot::chroot (const chroot& rhs):
@@ -131,11 +140,13 @@ sbuild::chroot::chroot (const chroot& rhs):
   root_groups(rhs.root_groups),
   aliases(rhs.aliases),
   preserve_environment(rhs.preserve_environment),
+  default_shell(rhs.default_shell),
   environment_filter(rhs.environment_filter),
   mount_location(rhs.mount_location),
   original(rhs.original),
   run_setup_scripts(rhs.run_setup_scripts),
   script_config(rhs.script_config),
+  profile(rhs.profile),
   command_prefix(rhs.command_prefix),
   message_verbosity(rhs.message_verbosity),
   facets()
@@ -164,6 +175,8 @@ sbuild::chroot::create (std::string const& type)
     new_chroot = new chroot_directory();
   else if (type == "plain")
     new_chroot = new chroot_plain();
+  else if (type == "custom")
+    new_chroot = new chroot_custom();
   else if (type == "file")
     new_chroot = new chroot_file();
 #ifdef SBUILD_FEATURE_BLOCKDEV
@@ -204,7 +217,7 @@ sbuild::chroot::set_name (std::string const& name)
   if (pos != std::string::npos)
     {
       error e(name, NAME_INVALID);
-      format fmt(_("Namespace separator '%1%' may not be used in a chroot name"));
+      format fmt(_("Namespace separator ‘%1%’ may not be used in a chroot name"));
       fmt % chroot_config::namespace_separator;
       e.set_reason(fmt.str());
       throw e;
@@ -312,7 +325,7 @@ sbuild::chroot::set_aliases (string_list const& aliases)
       if (found != std::string::npos)
 	{
 	  error e(*pos, NAME_INVALID);
-	  format fmt(_("Namespace separator '%1%' may not be used in an alias name"));
+	  format fmt(_("Namespace separator ‘%1%’ may not be used in an alias name"));
 	  fmt % chroot_config::namespace_separator;
 	  e.set_reason(fmt.str());
 	  throw e;
@@ -341,6 +354,18 @@ sbuild::chroot::set_preserve_environment (bool preserve_environment)
   this->preserve_environment = preserve_environment;
 }
 
+std::string const&
+sbuild::chroot::get_default_shell () const
+{
+  return this->default_shell;
+}
+
+void
+sbuild::chroot::set_default_shell (std::string const& default_shell)
+{
+  this->default_shell = default_shell;
+}
+
 regex const&
 sbuild::chroot::get_environment_filter () const
 {
@@ -387,6 +412,41 @@ void
 sbuild::chroot::set_script_config (std::string const& script_config)
 {
   this->script_config = script_config;
+
+  // Undo work of set_profile, so profile is completely unset.
+  this->profile.clear();
+  chroot_facet_userdata::ptr userdata =
+    get_facet<chroot_facet_userdata>();
+  if (userdata)
+    {
+      userdata->remove_data("setup.config");
+      userdata->remove_data("setup.copyfiles");
+      userdata->remove_data("setup.nssdatabases");
+      userdata->remove_data("setup.fstab");
+    }
+
+}
+
+std::string const&
+sbuild::chroot::get_profile () const
+{
+  return this->profile;
+}
+
+void
+sbuild::chroot::set_profile (std::string const& profile)
+{
+  this->profile = profile;
+
+  chroot_facet_userdata::ptr userdata =
+    get_facet<chroot_facet_userdata>();
+  if (userdata)
+    {
+      userdata->set_system_data("setup.config", this->profile + "/config");
+      userdata->set_system_data("setup.copyfiles", this->profile + "/copyfiles");
+      userdata->set_system_data("setup.nssdatabases", this->profile + "/nssdatabases");
+      userdata->set_system_data("setup.fstab", this->profile + "/fstab");
+    }
 }
 
 string_list const&
@@ -491,7 +551,13 @@ sbuild::chroot::setup_env (chroot const& chroot,
   env.add("CHROOT_DESCRIPTION", chroot.get_description());
   env.add("CHROOT_MOUNT_LOCATION", chroot.get_mount_location());
   env.add("CHROOT_PATH", chroot.get_path());
-  env.add("CHROOT_SCRIPT_CONFIG", normalname(std::string(SCHROOT_SYSCONF_DIR) +  '/' + chroot.get_script_config()));
+  if (!chroot.get_script_config().empty())
+    env.add("CHROOT_SCRIPT_CONFIG", normalname(std::string(SCHROOT_SYSCONF_DIR) +  '/' + chroot.get_script_config()));
+  if (!chroot.get_profile().empty())
+    {
+      env.add("CHROOT_PROFILE", chroot.get_profile());
+      env.add("CHROOT_PROFILE_DIR", normalname(std::string(SCHROOT_SYSCONF_DIR) +  '/' + chroot.get_profile()));
+    }
   env.add("CHROOT_SESSION_CREATE",
 	  static_cast<bool>(chroot.get_session_flags() & SESSION_CREATE));
   env.add("CHROOT_SESSION_CLONE",
@@ -608,8 +674,10 @@ sbuild::chroot::get_details (chroot const&  chroot,
     .add(_("Root Groups"), chroot.get_root_groups())
     .add(_("Aliases"), chroot.get_aliases())
     .add(_("Preserve Environment"), chroot.get_preserve_environment())
+    .add(_("Default Shell"), chroot.get_default_shell())
     .add(_("Environment Filter"), chroot.get_environment_filter())
     .add(_("Run Setup Scripts"), chroot.get_run_setup_scripts())
+    .add(_("Configuration Profile"), chroot.get_profile())
     .add(_("Script Configuration"), chroot.get_script_config())
     .add(_("Session Managed"),
 	 static_cast<bool>(chroot.get_session_flags() & chroot::SESSION_CREATE))
@@ -675,9 +743,14 @@ sbuild::chroot::get_keyfile (chroot const& chroot,
 			    keyfile, chroot.get_name(),
 			    "type");
 
-  keyfile::set_object_value(chroot, &chroot::get_script_config,
+  keyfile::set_object_value(chroot, &chroot::get_profile,
 			    keyfile, chroot.get_name(),
-			    "script-config");
+			    "profile");
+
+  if (!get_script_config().empty())
+    keyfile::set_object_value(chroot, &chroot::get_script_config,
+			      keyfile, chroot.get_name(),
+			      "script-config");
 
   keyfile::set_object_list_value(chroot, &chroot::get_aliases,
 				 keyfile, chroot.get_name(),
@@ -723,12 +796,17 @@ sbuild::chroot::get_keyfile (chroot const& chroot,
   keyfile::set_object_value(chroot, &chroot::get_preserve_environment,
 			    keyfile, chroot.get_name(),
 			    "preserve-environment");
+
+  keyfile::set_object_value(chroot, &chroot::get_default_shell,
+			    keyfile, chroot.get_name(),
+			    "shell");
 }
 
 void
-sbuild::chroot::set_keyfile (keyfile const& keyfile,
-			     string_list&   used_keys)
+sbuild::chroot::set_keyfile (keyfile const& keyfile)
 {
+  string_list used_keys;
+
   set_keyfile(*this, keyfile, used_keys);
 
   for (facet_list::const_iterator pos = facets.begin();
@@ -737,6 +815,69 @@ sbuild::chroot::set_keyfile (keyfile const& keyfile,
     {
       (*pos)->set_keyfile(*this, keyfile, used_keys);
     }
+
+  // Check for keys which weren't set above.  These may be either
+  // invalid keys or user-set keys.  The latter must have a namespace
+  // separated with one or more periods.  These may be later
+  // overridden by the user on the commandline.
+  {
+    std::string const& group = this->get_name();
+    const string_list total(keyfile.get_keys(group));
+
+    const string_set a(total.begin(), total.end());
+    const string_set b(used_keys.begin(), used_keys.end());
+
+    string_set unused;
+
+    set_difference(a.begin(), a.end(),
+		   b.begin(), b.end(),
+		   inserter(unused, unused.begin()));
+
+    string_map userdata_keys;
+    chroot_facet_userdata::ptr userdata =
+      get_facet<chroot_facet_userdata>();
+    for (string_set::const_iterator pos = unused.begin();
+	 pos != unused.end();
+	 ++pos)
+      {
+	if (userdata)
+	  {
+	    try
+	      {
+		std::string value;
+		if (keyfile.get_value(get_name(), *pos, value))
+		  userdata->set_data(*pos, value);
+	      }
+	    catch (std::runtime_error const& e)
+	      {
+		keyfile::size_type line = keyfile.get_line(group, *pos);
+		keyfile::error w(line, group, *pos,
+				 keyfile::PASSTHROUGH_LGK, e.what());
+
+		try
+		  {
+		    sbuild::error_base const& r =
+		      dynamic_cast<sbuild::error_base const&>(e);
+		    w.set_reason(r.get_reason());
+		  }
+		catch (...)
+		  {
+		  }
+		log_exception_warning(w);
+	      }
+	  }
+	else
+	  {
+	    // This should never happen since userdata should
+	    // always be present.  Note the error is duplicated in
+	    // the facet.
+	    keyfile::size_type line = keyfile.get_line(group, *pos);
+	    keyfile::error e(line, group, keyfile::INVALID_KEY, *pos);
+	    e.set_reason(_("This option is not valid for this chroot type"));
+	    log_exception_warning(e);
+	  }
+      }
+  }
 }
 
 void
@@ -790,10 +931,18 @@ sbuild::chroot::set_keyfile (chroot&        chroot,
 			    keyfile::PRIORITY_OBSOLETE);
   used_keys.push_back("run-exec-scripts");
 
+  keyfile::get_object_value(chroot, &chroot::set_profile,
+			    keyfile, chroot.get_name(),
+			    "profile",
+			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("profile");
+
   keyfile::get_object_value(chroot, &chroot::set_script_config,
 			    keyfile, chroot.get_name(),
 			    "script-config",
-			    keyfile::PRIORITY_OPTIONAL);
+			    session ?
+			    keyfile::PRIORITY_OPTIONAL :
+			    keyfile::PRIORITY_DEPRECATED);
   used_keys.push_back("script-config");
 
   keyfile::get_object_value(chroot, nullmethod,
@@ -879,4 +1028,10 @@ sbuild::chroot::set_keyfile (chroot&        chroot,
 			    "preserve-environment",
 			    keyfile::PRIORITY_OPTIONAL);
   used_keys.push_back("preserve-environment");
+
+  keyfile::get_object_value(chroot, &chroot::set_default_shell,
+			    keyfile, chroot.get_name(),
+			    "shell",
+			    keyfile::PRIORITY_OPTIONAL);
+  used_keys.push_back("shell");
 }
