@@ -91,6 +91,7 @@ namespace sbuild
 	FILE_UNLOCK,      ///< Failed to discard lock.
 	LOCATION_ABS,     ///< Location must have an absolute path.
 	NAME_INVALID,     ///< Invalid name.
+	SCRIPT_CONFIG_CV, ///< Could not set profile from script configuration path.
 	SESSION_UNLINK,   ///< Failed to unlink session file.
 	SESSION_WRITE,    ///< Failed to write session file.
 	VERBOSITY_INVALID ///< Message verbosity is invalid.
@@ -100,10 +101,10 @@ namespace sbuild
     typedef custom_error<error_code> error;
 
     /// A shared_ptr to a chroot object.
-    typedef std::tr1::shared_ptr<chroot> ptr;
+    typedef std::shared_ptr<chroot> ptr;
 
     /// A shared_ptr to a const chroot object.
-    typedef std::tr1::shared_ptr<const chroot> const_ptr;
+    typedef std::shared_ptr<const chroot> const_ptr;
 
   protected:
     /// The constructor.
@@ -322,6 +323,22 @@ namespace sbuild
     void
     set_preserve_environment (bool preserve_environment);
 
+    /**
+     * Get default shell.
+     *
+     * @returns default shell, or empty string if unset
+     */
+    std::string const&
+    get_default_shell () const;
+
+    /**
+     * Set the default shell.  This is the default interactive shell.
+     *
+     * @param default_shell the default shell.
+     */
+    void
+    set_default_shell (std::string const& default_shell);
+
     /**
      * Get the environment filter of the chroot.  This is a POSIX
      * extended regular expression used to remove insecure environment
@@ -406,6 +423,26 @@ namespace sbuild
     void
     set_script_config (std::string const& script_config);
 
+    /**
+     * Get the configuration profile for the chroot.  This is a
+     * directory, either relative to the configured pkgsysconfdir or
+     * an absolute path.
+     *
+     * @returns the configuration file name.
+     */
+    std::string const&
+    get_profile () const;
+
+    /**
+     * Set configuration profile for the chroot.  This is a directory,
+     * either relative to the configured pkgsysconfdir or an absolute
+     * path.
+     *
+     * @param profile the script configuration file.
+     */
+    void
+    set_profile (std::string const& profile);
+
     /**
      * Get the command_prefix for the chroot.  This is a command to
      * prefix to any command run in the chroot.
@@ -552,7 +589,7 @@ namespace sbuild
      * does not exist.
      */
     template <typename T>
-    std::tr1::shared_ptr<T>
+    std::shared_ptr<T>
     get_facet ();
 
     /**
@@ -563,7 +600,7 @@ namespace sbuild
      * does not exist.
      */
     template <typename T>
-    const std::tr1::shared_ptr<const T>
+    const std::shared_ptr<const T>
     get_facet () const;
 
     /**
@@ -573,7 +610,7 @@ namespace sbuild
      */
     template <typename T>
     void
-    add_facet (std::tr1::shared_ptr<T> facet);
+    add_facet (std::shared_ptr<T> facet);
 
     /**
      * Remove a chroot facet.  This is a templated method; use the
@@ -590,7 +627,7 @@ namespace sbuild
      */
     template <typename T>
     void
-    remove_facet (std::tr1::shared_ptr<T> facet);
+    remove_facet (std::shared_ptr<T> facet);
 
     /**
      * Replace an existing chroot facet with a new facet.
@@ -599,7 +636,7 @@ namespace sbuild
      */
     template <typename T>
     void
-    replace_facet (std::tr1::shared_ptr<T> facet);
+    replace_facet (std::shared_ptr<T> facet);
 
     /**
      * List all registered chroot facets.
@@ -657,9 +694,7 @@ namespace sbuild
     operator >> (keyfile const& keyfile,
 		 ptr&           rhs)
     {
-      string_list used;
-      rhs->set_keyfile(keyfile, used);
-      keyfile.check_keys(rhs->get_name(), used);
+      rhs->set_keyfile(keyfile);
       return keyfile;
     }
 
@@ -737,11 +772,9 @@ namespace sbuild
      * be determined.
      *
      * @param keyfile the keyfile to get the properties from.
-     * @param used_keys a list of the keys used will be set.
      */
     void
-    set_keyfile (keyfile const& keyfile,
-		 string_list&   used_keys);
+    set_keyfile (keyfile const& keyfile);
 
   protected:
     /**
@@ -775,6 +808,8 @@ namespace sbuild
     string_list   aliases;
     /// Preserve environment?
     bool          preserve_environment;
+    /// Default shell
+    std::string   default_shell;
     /// Environment filter regex.
     regex         environment_filter;
     /// Location to mount chroot in the filesystem (if any).
@@ -785,13 +820,15 @@ namespace sbuild
     bool          run_setup_scripts;
     /// Configuration of the setup and exec scripts.
     std::string   script_config;
+    /// Configuration profile for setup scripts (replaces script_config).
+    std::string   profile;
     /// Command prefix.
     string_list   command_prefix;
     /// The message verbosity.
     verbosity     message_verbosity;
 
     /// A shared pointer to a chroot facet.
-    typedef std::tr1::shared_ptr<chroot_facet> facet_ptr;
+    typedef std::shared_ptr<chroot_facet> facet_ptr;
     /// A list of chroot facets.
     typedef std::list<facet_ptr> facet_list;
     /// Contained chroot facets
@@ -834,16 +871,16 @@ namespace sbuild
 {
 
   template <typename T>
-  std::tr1::shared_ptr<T>
+  std::shared_ptr<T>
   chroot::get_facet ()
   {
-    std::tr1::shared_ptr<T> ret;
+    std::shared_ptr<T> ret;
 
     for (facet_list::const_iterator pos = facets.begin();
 	 pos != facets.end();
 	 ++pos)
       {
-	if (ret = std::tr1::dynamic_pointer_cast<T>(*pos))
+	if (ret = std::dynamic_pointer_cast<T>(*pos))
 	  break;
       }
 
@@ -851,27 +888,27 @@ namespace sbuild
   }
 
   template <typename T>
-  const std::tr1::shared_ptr<const T>
+  const std::shared_ptr<const T>
   chroot::get_facet () const
   {
-    std::tr1::shared_ptr<T> ret;
+    std::shared_ptr<T> ret;
 
     for (facet_list::const_iterator pos = facets.begin();
 	 pos != facets.end();
 	 ++pos)
       {
-	if (ret = std::tr1::dynamic_pointer_cast<T>(*pos))
+	if (ret = std::dynamic_pointer_cast<T>(*pos))
 	  break;
       }
 
-    return std::tr1::const_pointer_cast<T>(ret);
+    return std::const_pointer_cast<T>(ret);
   }
 
   template <typename T>
   void
-  chroot::add_facet (std::tr1::shared_ptr<T> facet)
+  chroot::add_facet (std::shared_ptr<T> facet)
   {
-    facet_ptr new_facet = std::tr1::dynamic_pointer_cast<chroot_facet>(facet);
+    facet_ptr new_facet = std::dynamic_pointer_cast<chroot_facet>(facet);
     if (!new_facet)
       throw error(FACET_INVALID);
 
@@ -879,7 +916,7 @@ namespace sbuild
 	 pos != facets.end();
 	 ++pos)
       {
-	if (std::tr1::dynamic_pointer_cast<T>(*pos))
+	if (std::dynamic_pointer_cast<T>(*pos))
 	  throw error(FACET_PRESENT);
       }
 
@@ -895,7 +932,7 @@ namespace sbuild
 	 pos != facets.end();
 	 ++pos)
       {
-	if (std::tr1::dynamic_pointer_cast<T>(*pos))
+	if (std::dynamic_pointer_cast<T>(*pos))
 	  {
 	    facets.erase(pos);
 	    break;
@@ -905,14 +942,14 @@ namespace sbuild
 
   template <typename T>
   void
-  chroot::remove_facet (std::tr1::shared_ptr<T> facet)
+  chroot::remove_facet (std::shared_ptr<T> facet)
   {
     remove_facet<T>();
   }
 
   template <typename T>
   void
-  chroot::replace_facet (std::tr1::shared_ptr<T> facet)
+  chroot::replace_facet (std::shared_ptr<T> facet)
   {
     remove_facet<T>();
     add_facet(facet);
