@@ -22,6 +22,7 @@
 #include "sbuild-chroot-facet-personality.h"
 #include "sbuild-chroot-facet-session.h"
 #include "sbuild-chroot-facet-session-clonable.h"
+#include "sbuild-chroot-facet-userdata.h"
 #ifdef SBUILD_FEATURE_PAM
 #include "sbuild-auth-pam.h"
 #include "sbuild-auth-pam-conv.h"
@@ -68,28 +69,28 @@ namespace
   emap init_errors[] =
     {
       // TRANSLATORS: %1% = directory
-      emap(session::CHDIR,          N_("Failed to change to directory '%1%'")),
+      emap(session::CHDIR,          N_("Failed to change to directory ‘%1%’")),
       // TRANSLATORS: %4% = directory
-      emap(session::CHDIR_FB,       N_("Falling back to directory '%4%'")),
+      emap(session::CHDIR_FB,       N_("Falling back to directory ‘%4%’")),
       emap(session::CHILD_CORE,     N_("Child dumped core")),
       emap(session::CHILD_FAIL,     N_("Child exited abnormally (reason unknown; not a signal or core dump)")),
       emap(session::CHILD_FORK,     N_("Failed to fork child")),
       // TRANSLATORS: %4% = signal name
-      emap(session::CHILD_SIGNAL,   N_("Child terminated by signal '%4%'")),
+      emap(session::CHILD_SIGNAL,   N_("Child terminated by signal ‘%4%’")),
       emap(session::CHILD_WAIT,     N_("Wait for child failed")),
       // TRANSLATORS: %1% = directory
-      emap(session::CHROOT,         N_("Failed to change root to directory '%1%'")),
+      emap(session::CHROOT,         N_("Failed to change root to directory ‘%1%’")),
       // TRANSLATORS: %1% = chroot name
-      emap(session::CHROOT_ALIAS,   N_("No chroot found matching name or alias '%1%'")),
+      emap(session::CHROOT_ALIAS,   N_("No chroot found matching name or alias ‘%1%’")),
       emap(session::CHROOT_LOCK,    N_("Failed to lock chroot")),
       emap(session::CHROOT_NOTFOUND,N_("%1%: Chroot not found")),
       emap(session::CHROOT_SETUP,   N_("Chroot setup failed")),
       // TRANSLATORS: %1% = chroot name
       emap(session::CHROOT_UNLOCK,  N_("Failed to unlock chroot")),
       // TRANSLATORS: %1% = command
-      emap(session::COMMAND_ABS,    N_("Command \"%1%\" must have an absolute path")),
+      emap(session::COMMAND_ABS,    N_("Command “%1%” must have an absolute path")),
       // TRANSLATORS: %1% = command
-      emap(session::EXEC,           N_("Failed to execute \"%1%\"")),
+      emap(session::EXEC,           N_("Failed to execute “%1%”")),
       // TRANSLATORS: A supplementary group is the list of additional
       // system groups a user belongs to, in addition to their default
       // group.
@@ -99,29 +100,29 @@ namespace
       // group.
       emap(session::GROUP_GET_SUPC, N_("Failed to get supplementary group count")),
       // TRANSLATORS: %1% = integer group ID
-      emap(session::GROUP_SET,      N_("Failed to set group '%1%'")),
+      emap(session::GROUP_SET,      N_("Failed to set group ‘%1%’")),
       emap(session::GROUP_SET_SUP,  N_("Failed to set supplementary groups")),
       // TRANSLATORS: %1% = group name
-      emap(session::GROUP_UNKNOWN,  N_("Group '%1%' not found")),
+      emap(session::GROUP_UNKNOWN,  N_("Group ‘%1%’ not found")),
       emap(session::PAM,            N_("PAM error")),
       emap(session::ROOT_DROP,      N_("Failed to drop root permissions")),
       // TRANSLATORS: %1% = chroot name
       // TRANSLATORS: %4% = session identifier
-      emap(session::SET_SESSION_ID, N_("%1%: Chroot does not support setting a session ID; ignoring session ID '%4%'")),
+      emap(session::SET_SESSION_ID, N_("%1%: Chroot does not support setting a session ID; ignoring session ID ‘%4%’")),
       // TRANSLATORS: %1% = command
-      emap(session::SHELL,          N_("Shell '%1%' not available")),
+      emap(session::SHELL,          N_("Shell ‘%1%’ not available")),
       // TRANSLATORS: %4% = command
-      emap(session::SHELL_FB,       N_("Falling back to shell '%4%'")),
+      emap(session::SHELL_FB,       N_("Falling back to shell ‘%4%’")),
       // TRANSLATORS: %4% = signal name
-      emap(session::SIGNAL_CATCH,   N_("Caught signal '%4%'")),
+      emap(session::SIGNAL_CATCH,   N_("Caught signal ‘%4%’")),
       // TRANSLATORS: %4% = signal name
-      emap(session::SIGNAL_SET,     N_("Failed to set signal handler '%4%'")),
+      emap(session::SIGNAL_SET,     N_("Failed to set signal handler ‘%4%’")),
       // TRANSLATORS: %1% = integer user ID
-      emap(session::USER_SET,       N_("Failed to set user '%1%'")),
+      emap(session::USER_SET,       N_("Failed to set user ‘%1%’")),
       // TRANSLATORS: %1% = user name
       // TRANSLATORS: %2% = user name
       // TRANSLATORS: Please translate "->" as a right arrow, e.g. U+2192
-      emap(session::USER_SWITCH,    N_("(%1%->%2%): User switching is not permitted")),
+      emap(session::USER_SWITCH,    N_("(%1%→%2%): User switching is not permitted")),
     };
 
   volatile bool sighup_called = false;
@@ -212,6 +213,8 @@ session::session (std::string const&  service,
   termios_ok(false),
   verbosity(),
   preserve_environment(false),
+  shell(),
+  user_options(),
   cwd(sbuild::getcwd())
 {
 }
@@ -292,6 +295,30 @@ session::set_preserve_environment (bool preserve_environment)
   this->preserve_environment = preserve_environment;
 }
 
+std::string const&
+session::get_shell_override () const
+{
+  return this->shell;
+}
+
+void
+session::set_shell_override (std::string const& shell)
+{
+  this->shell = shell;
+}
+
+string_map const&
+session::get_user_options () const
+{
+  return this->user_options;
+}
+
+void
+session::set_user_options (string_map const& user_options)
+{
+  this->user_options = user_options;
+}
+
 bool
 session::get_force () const
 {
@@ -364,15 +391,10 @@ session::is_group_member (std::string const& groupname) const
   if (!grp)
     {
       if (errno == 0)
-	{
-	  session::error e(groupname, session::GROUP_UNKNOWN);
-	  log_exception_warning(e);
-	}
+	log_debug(DEBUG_INFO) << "Group " << groupname << "not found" << endl;
       else
-	{
-	  session::error e(groupname, session::GROUP_UNKNOWN, strerror(errno));
-	  log_exception_warning(e);
-	}
+	log_debug(DEBUG_INFO) << "Group " << groupname
+			      << "not found: " << strerror(errno) << endl;
       return false;
     }
 
@@ -667,6 +689,22 @@ session::run_impl ()
 	  if (!this->verbosity.empty())
 	    chroot->set_verbosity(this->verbosity);
 
+	  // Set user options.
+	  chroot_facet_userdata::ptr userdata =
+	    chroot->get_facet<chroot_facet_userdata>();
+	  if (userdata)
+	    {
+	      // If the user running the command is root, or the user
+	      // being switched to is root, permit setting of
+	      // root-modifiable options in addition to
+	      // user-modifiable options.
+	      if (this->authstat->get_uid() == 0 ||
+		  this->authstat->get_ruid() == 0)
+		userdata->set_root_data(this->user_options);
+	      else
+		userdata->set_user_data(this->user_options);
+	    }
+
 	  // Following authentication success, default child status to
 	  // success so that operations such as beginning, ending and
 	  // recovering sessions will return success unless an
@@ -798,27 +836,79 @@ session::get_command_directories (sbuild::chroot::ptr& session_chroot,
   return ret;
 }
 
-std::string
-session::get_shell () const
+string_list
+session::get_shells (sbuild::chroot::ptr& session_chroot) const
 {
-  assert (!this->authstat->get_shell().empty());
-  std::string shell = this->authstat->get_shell();
+  string_list ret;
 
-  try
+  // Shell set with --shell (if any)
+  if (!this->shell.empty())
     {
-      stat(shell).check();
+      ret.push_back(this->shell);
     }
-  catch (std::runtime_error const& e)
+  else if (!session_chroot->get_default_shell().empty())
     {
-      error e1(shell, SHELL, e.what());
-      log_exception_warning(e1);
-
-      if (shell != "/bin/sh")
+      ret.push_back(session_chroot->get_default_shell());
+    }
+  else
+    {
+      if (get_preserve_environment())
 	{
-	  shell = "/bin/sh";
-	  error e2(SHELL_FB, shell);
-	  log_exception_warning(e2);
+	  // $SHELL (if --preserve-environment used)
+	  environment const& env(this->authstat->get_complete_environment());
+
+	  std::string envshell;
+	  if (env.get("SHELL", envshell) &&
+	      std::find(ret.begin(), ret.end(), envshell) == ret.end())
+	    ret.push_back(envshell);
 	}
+
+      // passwd pw_shell
+      std::string const& shell = this->authstat->get_shell();
+      if (!shell.empty())
+	ret.push_back(shell);
+
+      // Fallback nice interactive shell
+      if (std::find(ret.begin(), ret.end(), "/bin/bash") == ret.end())
+	ret.push_back("/bin/bash");
+
+      // Fallback basic interactive shell
+      if (std::find(ret.begin(), ret.end(), "/bin/sh") == ret.end())
+	ret.push_back("/bin/sh");
+    }
+
+  return ret;
+}
+
+std::string
+session::get_shell (sbuild::chroot::ptr& session_chroot) const
+{
+  string_list shells(get_shells(session_chroot));
+
+  std::string shell;
+
+  for (string_list::const_iterator pos = shells.begin();
+       pos != shells.end();
+       ++pos)
+    {
+      shell = *pos;
+
+      try
+	{
+	  stat(shell).check();
+	  break;
+	}
+      catch (std::runtime_error const& e)
+	{
+	  error e1(shell, SHELL, e.what());
+	  log_exception_warning(e1);
+	}
+    }
+
+  if (shell != *shells.begin())
+    {
+      error e2(SHELL_FB, shell);
+      log_exception_warning(e2);
     }
 
   return shell;
@@ -828,12 +918,12 @@ void
 session::get_command (sbuild::chroot::ptr& session_chroot,
 		      std::string&         file,
 		      string_list&         command,
-		      environment const&   env) const
+		      environment&         env) const
 {
   /* Run login shell */
   if (command.empty() ||
       command[0].empty()) // No command
-    get_login_command(session_chroot, file, command);
+    get_login_command(session_chroot, file, command, env);
   else
     get_user_command(session_chroot, file, command, env);
 }
@@ -841,12 +931,14 @@ session::get_command (sbuild::chroot::ptr& session_chroot,
 void
 session::get_login_command (sbuild::chroot::ptr& session_chroot,
 			    std::string&         file,
-			    string_list&         command) const
+			    string_list&         command,
+			    environment&         env) const
 {
   command.clear();
 
-  std::string shell = get_shell();
+  std::string shell = get_shell(session_chroot);
   file = shell;
+  env.add("SHELL", shell);
 
   bool login_shell =
     !(get_preserve_environment() ||
@@ -890,11 +982,11 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
 	  if (login_shell)
 	    // TRANSLATORS: %1% = chroot name
 	    // TRANSLATORS: %4% = command
-	    format_string = _("[%1% chroot] Running login shell: '%4%'");
+	    format_string = _("[%1% chroot] Running login shell: ‘%4%’");
 	  else
 	    // TRANSLATORS: %1% = chroot name
 	    // TRANSLATORS: %4% = command
-	    format_string = _("[%1% chroot] Running shell: '%4%'");
+	    format_string = _("[%1% chroot] Running shell: ‘%4%’");
 	}
       else
 	{
@@ -904,14 +996,14 @@ session::get_login_command (sbuild::chroot::ptr& session_chroot,
 	    // TRANSLATORS: %3% = user name
 	    // TRANSLATORS: %4% = command
 	    // TRANSLATORS: Please translate "->" as a right arrow, e.g. U+2192
-	    format_string = _("[%1% chroot] (%2%->%3%) Running login shell: '%4%'");
+	    format_string = _("[%1% chroot] (%2%→%3%) Running login shell: ‘%4%’");
 	  else
 	    // TRANSLATORS: %1% = chroot name
 	    // TRANSLATORS: %2% = user name
 	    // TRANSLATORS: %3% = user name
 	    // TRANSLATORS: %4% = command
 	    // TRANSLATORS: Please translate "->" as a right arrow, e.g. U+2192
-	    format_string = _("[%1% chroot] (%2%->%3%) Running shell: '%4%'");
+	    format_string = _("[%1% chroot] (%2%→%3%) Running shell: ‘%4%’");
 	}
 
       format fmt(format_string);
@@ -949,14 +1041,14 @@ session::get_user_command (sbuild::chroot::ptr& session_chroot,
       if (this->authstat->get_ruid() == this->authstat->get_uid())
 	// TRANSLATORS: %1% = chroot name
 	// TRANSLATORS: %4% = command
-	format_string = _("[%1% chroot] Running command: \"%4%\"");
+	format_string = _("[%1% chroot] Running command: “%4%”");
       else
 	// TRANSLATORS: %1% = chroot name
 	// TRANSLATORS: %2% = user name
 	// TRANSLATORS: %3% = user name
 	// TRANSLATORS: %4% = command
 	// TRANSLATORS: Please translate "->" as a right arrow, e.g. U+2192
-	format_string = (_("[%1% chroot] (%2%->%3%) Running command: \"%4%\""));
+	format_string = (_("[%1% chroot] (%2%→%3%) Running command: “%4%”"));
 
       format fmt(format_string);
       fmt % session_chroot->get_name()
@@ -1065,6 +1157,8 @@ session::setup_chroot (sbuild::chroot::ptr&       session_chroot,
   env.add("HOST_CPU", SBUILD_HOST_CPU);
   env.add("PLATFORM", SBUILD_PLATFORM);
 
+  env.add("PATH", "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin");
+
   run_parts rp(SCHROOT_CONF_SETUP_D,
 	       true, true, 022);
   rp.set_reverse(setup_type == chroot::SETUP_STOP);
@@ -1087,7 +1181,8 @@ session::setup_chroot (sbuild::chroot::ptr&       session_chroot,
 	  // The setup scripts don't use our syslog fd.
 	  closelog();
 
-	  chdir("/");
+	  if (chdir("/"))
+	    throw error("/", CHDIR, strerror(errno));
 	  /* This is required to ensure the scripts run with uid=0 and gid=0,
 	     otherwise setuid programs such as mount(8) will fail.  This
 	     should always succeed, because our euid=0 and egid=0.*/
@@ -1142,7 +1237,6 @@ session::run_child (sbuild::chroot::ptr& session_chroot)
   assert(!session_chroot->get_name().empty());
 
   assert(!this->authstat->get_user().empty());
-  assert(!get_shell().empty());
   assert(this->authstat->is_initialised()); // PAM must be initialised
 
   /* Set up environment */
