@@ -79,11 +79,16 @@ public:
   void setUp()
   {
     test_chroot_base<chroot_block_device>::setUp();
+    CPPUNIT_ASSERT(chroot);
     CPPUNIT_ASSERT(session);
     CPPUNIT_ASSERT(!source);
+    CPPUNIT_ASSERT(!session_source);
+#ifdef SBUILD_FEATURE_UNION
     CPPUNIT_ASSERT(chroot_union);
     CPPUNIT_ASSERT(session_union);
     CPPUNIT_ASSERT(source_union);
+    CPPUNIT_ASSERT(session_source_union);
+#endif // SBUILD_FEATURE_UNION
   }
 
   virtual void setup_chroot_props (sbuild::chroot::ptr& chroot)
@@ -136,12 +141,7 @@ public:
     expected.add("CHROOT_MOUNT_LOCATION", "/mnt/mount-location");
     expected.add("CHROOT_PATH",           "/mnt/mount-location/squeeze");
     expected.add("CHROOT_DEVICE",         "/dev/testdev");
-    expected.add("CHROOT_MOUNT_DEVICE",   "/dev/testdev");
     expected.add("CHROOT_MOUNT_OPTIONS",  "-t jfs -o quota,rw");
-
-#ifdef SBUILD_FEATURE_UNION
-    expected.add("CHROOT_UNION_TYPE",     "none");
-#endif // SBUILD_FEATURE_UNION
   }
 
   void test_setup_env()
@@ -152,6 +152,9 @@ public:
     expected.add("CHROOT_SESSION_CLONE",  "false");
     expected.add("CHROOT_SESSION_CREATE", "true");
     expected.add("CHROOT_SESSION_PURGE",  "false");
+#ifdef SBUILD_FEATURE_UNION
+    expected.add("CHROOT_UNION_TYPE",     "none");
+#endif // SBUILD_FEATURE_UNION
 
     test_chroot_base<chroot_block_device>::test_setup_env(chroot, expected);
   }
@@ -166,7 +169,10 @@ public:
     expected.add("CHROOT_SESSION_CLONE",  "false");
     expected.add("CHROOT_SESSION_CREATE", "false");
     expected.add("CHROOT_SESSION_PURGE",  "false");
+    expected.add("CHROOT_MOUNT_DEVICE",   "/dev/testdev");
+#ifdef SBUILD_FEATURE_UNION
     expected.add("CHROOT_UNION_TYPE",     "none");
+#endif // SBUILD_FEATURE_UNION
     test_chroot_base<chroot_block_device>::test_setup_env(session, expected);
   }
 
@@ -196,6 +202,7 @@ public:
     expected.add("CHROOT_SESSION_CLONE",  "false");
     expected.add("CHROOT_SESSION_CREATE", "false");
     expected.add("CHROOT_SESSION_PURGE",  "true");
+    expected.add("CHROOT_MOUNT_DEVICE",   "/dev/testdev");
     expected.add("CHROOT_UNION_TYPE",     "aufs");
     expected.add("CHROOT_UNION_MOUNT_OPTIONS",      "union-mount-options");
     expected.add("CHROOT_UNION_OVERLAY_DIRECTORY",  "/overlay/test-union-session-name");
@@ -232,8 +239,9 @@ public:
     std::string group = chroot->get_name();
     setup_keyfile_chroot(expected, group);
     setup_keyfile_block(expected, group);
+#ifdef SBUILD_FEATURE_UNION
     setup_keyfile_union_unconfigured(expected, group);
-
+#endif
     test_chroot_base<chroot_block_device>::test_setup_keyfile
       (chroot, expected, group);
   }
@@ -249,7 +257,9 @@ public:
     expected.set_value(group, "mount-device", "/dev/testdev");
     expected.set_value(group, "mount-location", "/mnt/mount-location");
     setup_keyfile_session_clone(expected, group);
+#ifdef SBUILD_FEATURE_UNION
     setup_keyfile_union_unconfigured(expected, group);
+#endif
 
     test_chroot_base<chroot_block_device>::test_setup_keyfile
       (session, expected, group);
@@ -294,7 +304,6 @@ public:
     setup_keyfile_source_clone(expected, group);
     setup_keyfile_block(expected, group);
     expected.set_value(group, "description", chroot->get_description() + ' ' + _("(source chroot)"));
-    setup_keyfile_union_unconfigured(expected, group);
 
     test_chroot_base<chroot_block_device>::test_setup_keyfile
       (source_union, expected, group);
